﻿using Kivii.Client.SampleAssigns;
using Kivii.Client.SampleInbounds.Entities;
using Kivii.Configuration;
using Kivii.Linq;
using Kivii.Samples.Entities;
using Kivii.Samples.Transforms;
using Readers;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace Kivii.Client.SampleInbounds
{
    public partial class FrmMain : Form
    {
        #region 私有变量
        private JsonServiceClient _client;
        private Location _currentLocation;
        private Sample _currentPackage;
        private List<Sample> _currentPackageSamples;
        private List<Sample> _currentScanSamples;
        private List<Organization> _currentOrgs;
        private AuthenticateResponse _userInfo;

        private Reader reader;
        private ReaderSetting currentSetting;
        private InventoryBuffer currentInventoryBuffer;
        private string defaultPort = "COM1";
        private int defaultBaudrate = 115200;

        private bool isConnected = false;
        //盘存操作前，需要先设置工作天线，用于标识当前是否在执行盘存操作
        private bool m_bInventory = false;
        private List<string> _currentTotalEpcs;//每次启动交接读出来的epc号集合,每次重启前会清空
        #endregion

        public FrmMain()
        {
            InitializeComponent();
        }

        private void FrmMain_Load(object sender, EventArgs e)
        {
            WindowState = FormWindowState.Maximized;
            initControls();
            loadDefault(tbxServiceUrl, tbxUserName, tbxPassword);
        }

        #region 界面控件默认值
        private void initControls()
        {
            dgvSamples.AutoGenerateColumns = false;
            dgvRfidSamples.AutoGenerateColumns = false;
            _currentLocation = null;
            _currentPackage = null;
            _currentPackageSamples = null;
            _currentScanSamples = null;
            _currentTotalEpcs = null;
            _userInfo = null;
            tbxServiceUrl.Enabled = true;
            tbxUserName.Enabled = true;
            tbxPassword.Enabled = true;
            tbSetCurrentPackage.Enabled = false;
            btnLogin.Enabled = true;
            btnLogout.Enabled = false;
            btnScanRfid.Enabled = false;
            btnSampleAssign.Enabled = false;
            tbScanSample.Enabled = false;
            tbScanSample.Text = "";
            btnClearCurrentPackage.Enabled = false;
            tbCurrentLocation.Text = "";
            lbCurrentPackageStatus.Text = "";
            cbSetCurrentPackageOwner.DataSource = null;
            cbSetCurrentPackageOwner.DisplayMember = "FullName";
            cbSetCurrentPackageOwner.ValueMember = "Kvid";
            tbCurrentPackage.Text = "";
            tbSetCurrentPackage.Text = "";
            dgvSamples.DataSource = null;
            dgvRfidSamples.DataSource = null;
        }

        private void loadDefault(params Control[] ctls)
        {
            try
            {
                using (var conn = KiviiContext.GetOpenedDbConnection<UiSetting>())
                {
                    foreach (var ctl in ctls)
                    {
                        //UiSetting setting = null;
                        var setting = conn.Select<UiSetting>(o => o.FormName == this.Name && o.ControlName == ctl.Name).FirstOrDefault();
                        if (setting == null)
                        {
                            if (ctl.Name == tbxServiceUrl.Name) ctl.Text = Configs.DefaultBaseUrl;
                            continue;
                        }
                        ctl.Text = setting.ControlValue;
                    }
                }
            }
            catch
            {

            }
        }

        private void setDefault(params Control[] ctls)
        {
            try
            {
                using (var conn = KiviiContext.GetOpenedDbConnection<UiSetting>())
                {
                    foreach (var ctl in ctls)
                    {
                        var setting = conn.Select<UiSetting>(o => o.FormName == this.Name && o.ControlName == ctl.Name).FirstOrDefault();
                        if (setting == null)
                        {
                            setting = new UiSetting();
                            setting.FormName = this.Name;
                            setting.ControlName = ctl.Name;
                            setting.ControlValue = ctl.Text;
                            conn.Insert(setting);
                        }
                        else
                        {
                            setting.ControlValue = ctl.Text;
                            conn.Update(setting);
                        }
                    }
                }
            }
            catch
            {

            }
        }

        /// <summary>
        /// 输出消息
        /// </summary>
        /// <param name="message">消息内容</param>
        /// <param name="clearMessage">是否清除消息</param>
        private void outputMessage(string message, bool clearMessage = false)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action(() => outputMessage(message, clearMessage)));
                return;
            }
            if (clearMessage) rtbMessage.Clear();
            if (rtbMessage.Lines.Count() > 500) rtbMessage.Clear();
            var time = DateTime.Now.ToString("HH:mm:ss");
            rtbMessage.AppendText($"[{time}]{message}{Environment.NewLine}");
            rtbMessage.ScrollToCaret();
        }
        #endregion

        #region Reader Function
        private bool connectReader()
        {
            reader = new Reader();
            currentSetting = new ReaderSetting();
            currentInventoryBuffer = new InventoryBuffer();

            //设置回调方法
            reader.AnalyCallback = AnalyData;
            reader.GetFrequencyRegion(currentSetting.btReadId);
            Thread.Sleep(500);

            currentInventoryBuffer.ClearInventoryPar();
            currentInventoryBuffer.btRepeat = Convert.ToByte(1);//设置每条命令盘存次数 1次
            currentInventoryBuffer.bLoopCustomizedSession = false;
            currentInventoryBuffer.lAntenna.Add(0x00);//设置天线1
            currentInventoryBuffer.lAntenna.Add(0x01);//设置天线2
            currentInventoryBuffer.lAntenna.Add(0x02);//设置天线3
            currentInventoryBuffer.lAntenna.Add(0x03);//设置天线4

            string exception = string.Empty;
            int nRet = reader.OpenCom(defaultPort, defaultBaudrate, out exception);
            if (nRet != 0)
            {
                isConnected = false;
                outputMessage("连接读写器失败，失败原因： " + exception);
                return false;
            }
            else
            {
                isConnected = true;
                outputMessage($"当前连接: {Environment.NewLine}{defaultPort}@{defaultBaudrate}");
                return true;
            }
        }
        private void disConnectReader()
        {
            //处理串口断开连接读写器
            if (reader != null) reader.CloseCom();
            reader = null;
            isConnected = false;
            return;
        }
        public void AnalyData(MessageTran msgTran)
        {
            if (msgTran.PacketType != 0xA0)
            {
                return;
            }
            switch (msgTran.Cmd)
            {
                //case 0x81:
                //    var tid = ReaderExtension.ProcessReadTag(msgTran, reader, currentInventoryBuffer, currentSetting);
                //    samplePreAssigning(tid);
                //    break;
                case 0x89://读取Epc
                    var epc = ReaderExtension.ProcessInventoryReal(msgTran, reader, currentInventoryBuffer, currentSetting);
                    if (epc.IsNullOrEmpty()) return;
                    if (_currentTotalEpcs.IsNullOrEmpty()) _currentTotalEpcs = new List<string>();
                    if (!_currentTotalEpcs.Contains(epc))
                    {
                        _currentTotalEpcs.Add(epc);
                        samplePreAssigning(null, epc);
                    }
                    break;
                case 0x74:
                    ReaderExtension.ProcessSetWorkAntenna(msgTran, reader, currentInventoryBuffer, currentSetting, m_bInventory);
                    break;
            }
        }

        public void btnRealTimeInventory_Start()
        {
            this.Invoke(new Action(delegate
            {
                dgvRfidSamples.DataSource = null;
                btnScanRfid.Text = "停止扫描";
                btnScanRfid.BackColor = Color.DarkGray;
                btnScanRfid.ForeColor = Color.Black;
            }));
            outputMessage("开始扫描中...");
            _currentTotalEpcs = new List<string>();//每次启动交接就要把当前epc号全部清空
            _currentScanSamples = null;
            m_bInventory = true;
            currentInventoryBuffer.bLoopInventory = true;
            currentInventoryBuffer.bLoopInventoryReal = true;
            currentInventoryBuffer.ClearInventoryRealResult();
            byte btWorkAntenna = currentInventoryBuffer.lAntenna[currentInventoryBuffer.nIndexAntenna];
            reader.SetWorkAntenna(currentSetting.btReadId, btWorkAntenna);
            currentSetting.btWorkAntenna = btWorkAntenna;
            return;
        }
        public void btnRealTimeInventory_Stop()
        {
            m_bInventory = false;
            currentInventoryBuffer.bLoopInventory = false;
            this.Invoke(new Action(delegate
            {
                btnScanRfid.Text = "开始扫描";
                btnScanRfid.BackColor = Color.PapayaWhip;
                btnScanRfid.ForeColor = Color.Chocolate;
            }));
            disConnectReader();
            outputMessage("停止扫描");
            return;
        }
        #endregion

        #region 业务方法
        private void initUserInfo(AuthenticateResponse response = null)
        {
            if (response == null)
            {
                _userInfo = null;
                _currentLocation = null;
                _currentPackage = null;
                return;
            }
            _userInfo = new AuthenticateResponse();
            _userInfo.PopulateWith(response);
            //tbCurrentPackageDepartment.Text = _userInfo.DepartmentName;
            initLocation();
            initOrganizations();
        }

        private void initOrganizations()
        {
            if (_client == null)
            {
                MessageBox.Show("请先登录!");
                return;
            }
            var request = new OrganizationQuery();
            var task = _client.PostAsync(request);
            task.Success(resp =>
            {
                if (resp.Results.Count <= 0)
                {
                    MessageBox.Show("未能获取所属部门信息!", "初始化实验室组");
                    return;
                }
                this.Invoke(new Action(delegate
                {
                    cbSetCurrentPackageOwner.DataSource = null;

                }));
                _currentOrgs = resp.Results.ToList();
                int indexWuli = _currentOrgs.FindIndex(o => o.FullName.Contains("物理"));
                this.Invoke(new Action(delegate
                {
                    cbSetCurrentPackageOwner.Enabled = true;
                    cbSetCurrentPackageOwner.DataSource = _currentOrgs;
                    if (indexWuli != -1) cbSetCurrentPackageOwner.SelectedIndex = indexWuli;
                }));
            });
            task.Error(err =>
            {
                outputMessage(err.GetResponseStatus().Message);
            });
        }
        private void initLocation()
        {
            if (_client == null)
            {
                MessageBox.Show("请先登录!");
                return;
            }
            if (_userInfo == null)
            {
                MessageBox.Show("未取得登录人信息,请重新登录!");
                return;
            }
            var request = new LocationQuery();
            request.OwnerKvid = _userInfo.DepartmentKvid;
            //request.OwnerKvid = Guid.Parse("5F344646-C9D2-413F-8DC6-500265C01CA2");//仓库
            //request.OwnerKvid = Guid.Parse("D4CE47D2-5FC8-479D-A0CA-BACDF1A891BF");//色牢度组
            //request.OwnerKvid = Guid.Parse("9D026818-6E78-40F8-A715-2887F96F34DC");//纤检组
            //request.OwnerKvid = Guid.Parse("208C24FB-1443-4B9E-9886-02ED3B9A8F45");//安全组
            //request.OwnerKvid = Guid.Parse("7CF9CF7B-19F5-4A1A-A94C-5BC21EA8ED76");//纤维含量组
            //request.OwnerKvid = Guid.Parse("BB355D5A-9D4E-4337-B577-7BB7A1625F68");//羽绒组
            var task = _client.PostAsync(request);
            task.Success(resp =>
            {
                if (resp.Results.Count <= 0)
                {
                    MessageBox.Show("未找到当前登录人所在组及位置信息!", "位置信息");
                    return;
                }
                var location = resp.Results.FirstOrDefault();
                _currentLocation = new Location();
                _currentLocation.PopulateWith(location);
                tbSetCurrentPackage.Enabled = _currentLocation != null;
                tbCurrentLocation.Text = _currentLocation.Address;
                btnScanRfid.Enabled = _currentLocation != null;
                tbScanSample.Enabled = _currentLocation != null;
                tbScanSample.Text = "";
            });
            task.Error(err =>
            {
                _currentLocation = null;
                tbSetCurrentPackage.Enabled = _currentLocation != null;
                MessageBox.Show(err.Message, "位置信息");
            });
        }
        private void setCurrentPackage(Sample package)
        {
            _currentPackage = new Sample();
            _currentPackage.PopulateWith(package);
            tbSetCurrentPackage.Text = _currentPackage.Name;
            tbSetCurrentPackage.Enabled = _currentPackage == null;
            tbCurrentPackage.Text = _currentPackage.Name + (_currentPackage.PackageName.IsNullOrEmpty() ? "" : $"({_currentPackage.PackageName})");
            btnClearCurrentPackage.Enabled = _currentPackage != null;
            //cbSetCurrentPackageNode.Enabled = _currentPackage.Status == (int)PackageStatus.Empty || _currentPackage.Status == (int)PackageStatus.Full;
            //btnScanRfid.Enabled = _currentPackage.Status == (int)PackageStatus.Full;
            btnSampleAssign.Enabled = _currentPackage.Status != (int)PackageStatus.Inbound;
            //tbScanSample.Enabled = _currentPackage.Status == (int)PackageStatus.Full;
            //tbScanSample.Text = "";
            if (_currentPackage.Status != (int)PackageStatus.Empty)
            {
                querySampleInPackaged();
            }
            if (_currentPackage.Status == (int)PackageStatus.Inbound)
            {
                lbCurrentPackageStatus.Text = "当前箱子已入库";
                outputMessage($"当前箱子为已入库状态,无法继续操作!");
                return;
            }
            Thread.Sleep(200);
            if (!isConnected || reader == null)
            {
                btnScanRfid_Click(null, null);
            }
        }

        private void searchPackage(string packageName = null, Guid? packageKvid = null)
        {
            if (_client == null)
            {
                MessageBox.Show("请先登录!");
                return;
            }
            if (packageName.IsNullOrEmpty() && (packageKvid == null || packageKvid == Guid.Empty))
            {
                MessageBox.Show("请输入箱号!");
                return;
            }
            var request = new SampleQuery();
            if (!packageName.IsNullOrEmpty()) request.Name = packageName;
            if (packageKvid != null && packageKvid != Guid.Empty) request.Kvid = packageKvid.Value;
            request.Type = SampleType.Package;
            var task = _client.PostAsync(request);
            task.Success(resp =>
            {
                if (resp.Results.Count <= 0)
                {
                    _currentPackage = null;
                    MessageBox.Show("未找到箱号!", "箱号设置");
                    return;
                }
                var package = resp.Results.FirstOrDefault();
                setCurrentPackage(package);
            });
            task.Error(err =>
            {
                _currentPackage = null;
                tbSetCurrentPackage.Enabled = true;
                tbCurrentPackage.Text = "";
                btnClearCurrentPackage.Enabled = false;
                //cbSetCurrentPackageNode.SelectedItem = null;
                //cbSetCurrentPackageNode.Enabled = false;
                btnScanRfid.Enabled = false;
                tbScanSample.Enabled = false;
                tbScanSample.Text = "";
                outputMessage($"查询箱号错误!原因:{err.GetResponseStatus().Message}");
            });
        }

        private void querySampleInPackaged()
        {
            if (_client == null)
            {
                MessageBox.Show("请先登录!");
                return;
            }
            if (_currentPackage == null)
            {
                MessageBox.Show("请先设置箱号!");
                return;
            }
            this.Invoke(new Action(delegate
            {
                dgvSamples.DataSource = null;
                lbPackageSamplesCount.Text = "0";
                lbPackageSampleMissCount.Text = "0";
            }));
            var request = new SampleQuery2();
            request.PackageKvid = _currentPackage.Kvid;
            var task = _client.PostAsync(request);
            task.Success(resp =>
            {
                int index = cbSetCurrentPackageOwner.SelectedIndex;
                if (resp.Results.Count > 0)
                {
                    var first = resp.Results[0];
                    if (!_currentOrgs.IsNullOrEmpty())
                    {
                        index = _currentOrgs.FindIndex(o => o.Kvid == first.OwnerKvid);
                    }
                }
                _currentPackageSamples = resp.Results.ToList();
                if (!_currentScanSamples.IsNullOrEmpty())
                {
                    foreach (var item in _currentPackageSamples)
                    {
                        if (_currentScanSamples.Exists(o => o.Kvid == item.Kvid))
                        {
                            item.IsMatched = true;
                        }
                    }
                }
                this.Invoke(new Action(delegate
                {
                    if (index != -1) cbSetCurrentPackageOwner.SelectedIndex = index;
                    dgvSamples.DataSource = _currentPackageSamples;
                    lbPackageSamplesCount.Text = _currentPackageSamples.Count.ToString();
                    lbPackageSampleMissCount.Text = _currentPackageSamples.Where(o => !o.IsMatched).Count().ToString();
                }));
                outputMessage($"成功查询到箱号:{_currentPackage.Name}中{_currentPackageSamples.Count}件样品!");
            });
            task.Error(err =>
            {
                outputMessage($"查询样品错误!原因:{err.GetResponseStatus().Message}");
                //MessageBox.Show(err.Message, "查询样品");
            });
        }

        private void samplePreAssigning(string tid = null, string epc = null, string bizId = null)
        {
            if (tid.IsNullOrEmpty() && bizId.IsNullOrEmpty() && epc.IsNullOrEmpty()) return;
            var request = new SamplePrePackaging();
            if (!tid.IsNullOrEmpty()) request.Tid = tid;
            if (!bizId.IsNullOrEmpty()) request.BizId = bizId;
            if(!Guid.TryParse(epc, out var epcKvid))
            {
                return;
                //request.BizKvid = epcKvid;
            }
            else
            {
                request.BizKvid = epcKvid;
            }
            var ownerSelected = cbSetCurrentPackageOwner.SelectedItem as Organization;
            if (ownerSelected == null)
            {
                outputMessage("请设置交接组部门信息!");
                return;
            }
            request.OwnerKvid = ownerSelected.Kvid;
            try
            {
                var resp = _client.Post(request);
                if (resp.Results.Count <= 0)
                {
                    //MessageBox.Show("未找到样品,当前组不包含此样品!", "交接查询");
                    outputMessage($"未找到样品:{epc},当前组不包含此样品!");
                    return;
                }
                var record = resp.Results[0];
                if (_currentPackage == null && record.PackageKvid != Guid.Empty)
                {
                    searchPackage(null, record.PackageKvid);
                }
                this.Invoke(new Action(delegate
                {
                    dgvRfidSamples.DataSource = null;
                    lbScanSamplesCount.Text = "0";
                    lbScanSampleMoreCount.Text = "0";
                }));
                if (_currentScanSamples == null) _currentScanSamples = new List<Sample>();
                if (!_currentPackageSamples.IsNullOrEmpty())
                {
                    this.Invoke(new Action(delegate
                    {
                        dgvSamples.DataSource = null;
                    }));
                    foreach (var item in _currentPackageSamples)
                    {
                        if (item.Kvid == record.Kvid)
                        {
                            item.IsMatched = true;
                            record.IsMatched = true;
                        }
                    }
                    this.Invoke(new Action(delegate
                    {
                        dgvSamples.DataSource = _currentPackageSamples;
                        lbPackageSamplesCount.Text = _currentPackageSamples.Count.ToString();
                        lbPackageSampleMissCount.Text = _currentPackageSamples.Where(o => !o.IsMatched).Count().ToString();
                    }));
                }
                if (!_currentScanSamples.Exists(o => o.Kvid == record.Kvid))
                {
                    //record.IsSelected = true;
                    if (ckbSelectAll.Checked && record.PackageKvid == Guid.Empty) record.IsSelected = true;
                    _currentScanSamples.Add(record);
                    _currentScanSamples = _currentScanSamples.OrderBy(o => o.PackageKvid).ThenBy(o => o.BizId).ToList();
                    outputMessage($"扫描结果增加：报告号[{record.BizId}]");
                }
                this.Invoke(new Action(delegate
                {
                    dgvRfidSamples.DataSource = _currentScanSamples;
                    lbScanSamplesCount.Text = _currentScanSamples.Count.ToString();
                    lbScanSampleMoreCount.Text = _currentScanSamples.Where(o => !o.IsMatched).Count().ToString();
                }));
                //sampleAssigning(record);
            }
            catch (Exception ex)
            {
                outputMessage($"交接查询错误,{ex.GetResponseStatus().Message}");
            }
        }

        private void samplePackaging(List<Guid> kvids)
        {
            var request = new SamplePackaging();
            request.Package = _currentPackage;
            request.Kvids = kvids;
            request.Location = _currentLocation;
            var rtns = new List<Sample>();
            //var task = _client.PostAsync(request);
            try
            {
                var resp = _client.Post(request);
                if (resp.Results.Count <= 0)
                {
                    return;
                }
                if (_currentScanSamples == null) _currentScanSamples = new List<Sample>();
                foreach (var record in resp.Results)
                {
                    if (_currentScanSamples.Exists(o => o.Kvid == record.Kvid))
                    {
                        _currentScanSamples.RemoveAll(o => o.Kvid == record.Kvid);
                        outputMessage($"{record.BizId}样品:[{record.Name}]在{_currentLocation.Address}处入箱完成.");
                    }
                    this.Invoke(new Action(delegate
                    {
                        dgvRfidSamples.DataSource = null;
                    }));
                    this.Invoke(new Action(delegate
                    {
                        dgvRfidSamples.DataSource = _currentScanSamples;
                    }));
                }
                //rtns.AddRange(resp.Results);
            }
            catch (Exception ex)
            {
                outputMessage($"入箱错误,原因:{ex.GetResponseStatus().Message}");
            }
        }

        private void sampleAssigning(List<Guid> kvids)
        {
            var request = new SampleAssign();
            request.Kvids = kvids;
            request.Location = _currentLocation;
            //var task = _client.PostAsync(request);
            try
            {
                var resp = _client.Post(request);
                if (resp.Results.Count <= 0)
                {
                    return;
                }
                if (_currentScanSamples == null) _currentScanSamples = new List<Sample>();
                foreach (var record in resp.Results)
                {
                    if (_currentScanSamples.Exists(o => o.Kvid == record.Kvid))
                    {
                        _currentScanSamples.RemoveAll(o => o.Kvid == record.Kvid);
                        outputMessage($"{record.BizId}样品:[{record.Name}]交接至{_currentLocation.Address}完成.");
                    }
                    this.Invoke(new Action(delegate
                    {
                        dgvRfidSamples.DataSource = null;
                    }));
                    this.Invoke(new Action(delegate
                    {
                        dgvRfidSamples.DataSource = _currentScanSamples;
                    }));
                }
                
            }
            catch (Exception ex)
            {
                outputMessage($"交接错误,原因:{ex.GetResponseStatus().Message}");
                //MessageBox.Show(ex.GetResponseStatus().Message, "样品入箱");
            }
        }
        private void preSampleAssigning(List<Sample> samples)
        {
            if (samples.IsNullOrEmpty()) return;
            var kvids = new List<Guid>();
            var unPackageKvids = new List<Guid>();
            var notCurrentPackageKvids = new List<Guid>();
            foreach (var sample in samples)//过滤重复交接的
            {
                if (sample.PackageKvid == Guid.Empty)//扫描的样品并未在设置的包裹里
                {
                    if (sample.IsSelected) unPackageKvids.Add(sample.Kvid);
                    continue;
                }
                if (sample.PackageKvid != _currentPackage.Kvid)
                {
                    notCurrentPackageKvids.Add(sample.Kvid);
                    continue;
                }
                if (sample.CurrentLocation == null)
                {
                    this.Invoke(new Action(delegate
                    {
                        dgvRfidSamples.DataSource = null;
                    }));
                    _currentScanSamples.RemoveAll(o => o.Kvid == sample.Kvid);
                    outputMessage($"{sample.BizId}样品:[{sample.Name}]无地点信息,已移除交接.");

                    this.Invoke(new Action(delegate
                    {
                        dgvRfidSamples.DataSource = _currentScanSamples;
                    }));
                    continue;
                }
                if (sample.CurrentLocation.InternalCode == _currentLocation.InternalCode)
                {
                    this.Invoke(new Action(delegate
                    {
                        dgvRfidSamples.DataSource = null;
                    }));
                    _currentScanSamples.RemoveAll(o => o.Kvid == sample.Kvid);
                    outputMessage($"{sample.BizId}样品:[{sample.Name}]已经在仓库,重复交接,已自动移除.");

                    this.Invoke(new Action(delegate
                    {
                        dgvRfidSamples.DataSource = _currentScanSamples;
                    }));
                    continue;
                }
                kvids.Add(sample.Kvid);
            }
            if (!kvids.IsNullOrEmpty())
            {
                sampleAssigning(kvids);
            }
            if (!unPackageKvids.IsNullOrEmpty())
            {
                if (ckbAutoPackage.Checked) samplePackaging(unPackageKvids);
            }
        }
        #endregion

        private void btnLogin_Click(object sender, EventArgs e)
        {
            if (_client != null)
            {
                MessageBox.Show("登陆前先退出原连接");
                return;
            }
            btnLogin.Enabled = false;
            _client = new JsonServiceClient(tbxServiceUrl.Text);
            var auth = new Authenticate { UserName = tbxUserName.Text, Password = tbxPassword.Text, provider = "Kivii" };
            var task = _client.PostAsync(auth);
            task.Success(resp =>
            {
                initUserInfo(resp);
                btnLogin.Enabled = false;
                btnLogout.Enabled = !btnLogin.Enabled;
                tbxServiceUrl.Enabled = btnLogin.Enabled;
                tbxUserName.Enabled = btnLogin.Enabled;
                tbxPassword.Enabled = btnLogin.Enabled;
                //tbSetCurrentPackage.Enabled = !btnLogin.Enabled;
                setDefault(tbxServiceUrl, tbxUserName, tbxPassword);
                outputMessage($"登录成功,登录人:{resp.DisplayName}", true);
                connectReader();
            }, true);
            task.Error(err =>
            {
                btnLogin.Enabled = true;
                initUserInfo();
                _client.Dispose();
                _client = null;
                outputMessage($"登录失败,原因:{err.GetResponseStatus().Message}");
            }, true);
        }

        private void btnLogout_Click(object sender, EventArgs e)
        {
            if (_client == null)
            {
                MessageBox.Show("未连接");
                return;
            }
            btnLogout.Enabled = false;
            var auth = new Authenticate { provider = "Logout" };
            var task = _client.PostAsync(auth);
            task.Success(resp =>
            {
                initControls();
                _client.Dispose();
                _client = null;
                initUserInfo();
                outputMessage($"退出成功");
            });
            task.Error(err =>
            {
                btnLogout.Enabled = true;
                MessageBox.Show(err.Message, "Logout Error");
                outputMessage($"退出失败!原因:{err.GetResponseStatus().Message}");
            });
        }

        private void tbxPassword_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                btnLogin_Click(null, null);
            }
        }


        private void tbSetCurrentPackage_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                searchPackage(tbSetCurrentPackage.Text);
            }
        }

        private void btnClearCurrentPackage_Click(object sender, EventArgs e)
        {
            _currentPackage = null;
            _currentPackageSamples = null;
            _currentScanSamples = null;
            tbSetCurrentPackage.Enabled = true;
            tbCurrentPackage.Text = "";
            btnClearCurrentPackage.Enabled = false;
            //btnScanRfid.Enabled = false;
            btnSampleAssign.Enabled = false;
            lbPackageSamplesCount.Text = "0";
            lbPackageSampleMissCount.Text = "0";
            lbScanSamplesCount.Text = "0";
            lbScanSampleMoreCount.Text = "0";
            //tbScanSample.Enabled = false;
            //tbScanSample.Text = "";
            lbCurrentPackageStatus.Text = "";
            //cbSetCurrentPackageOwner.DataSource = null;
            //cbSetCurrentPackageOwner.DisplayMember = "OwnerName";
            //cbSetCurrentPackageOwner.ValueMember = "OwnerKvid";
            //cbSetCurrentPackageOwner.Enabled = false;
            dgvSamples.DataSource = null;
            dgvRfidSamples.DataSource = null;
            btnRealTimeInventory_Stop();
            outputMessage($"箱号已重设!");
        }

        private void dgvSamples_RowPostPaint(object sender, DataGridViewRowPostPaintEventArgs e)
        {
            dgvSamples.TopLeftHeaderCell.Value = "序号";
            //自动编号，与数据无关
            Rectangle rectangle = new Rectangle(e.RowBounds.Location.X,
               e.RowBounds.Location.Y,
               dgvSamples.RowHeadersWidth - 4,
               e.RowBounds.Height);
            TextRenderer.DrawText(e.Graphics,
                  (e.RowIndex + 1).ToString(),
                   dgvSamples.RowHeadersDefaultCellStyle.Font,
                   rectangle,
                   dgvSamples.RowHeadersDefaultCellStyle.ForeColor,
                   TextFormatFlags.VerticalCenter | TextFormatFlags.Right);
        }

        private void dgvRfidSamples_RowPostPaint(object sender, DataGridViewRowPostPaintEventArgs e)
        {
            dgvRfidSamples.TopLeftHeaderCell.Value = "序号";
            //自动编号，与数据无关
            Rectangle rectangle = new Rectangle(e.RowBounds.Location.X,
               e.RowBounds.Location.Y,
               dgvRfidSamples.RowHeadersWidth - 4,
               e.RowBounds.Height);
            TextRenderer.DrawText(e.Graphics,
                  (e.RowIndex + 1).ToString(),
                   dgvRfidSamples.RowHeadersDefaultCellStyle.Font,
                   rectangle,
                   dgvRfidSamples.RowHeadersDefaultCellStyle.ForeColor,
                   TextFormatFlags.VerticalCenter | TextFormatFlags.Right);
        }

        private void cbSetCurrentPackageOwner_SelectedValueChanged(object sender, EventArgs e)
        {
            var selected = cbSetCurrentPackageOwner.SelectedItem as Organization;
            if (selected != null) outputMessage($"当前设置交接组:{selected.FullName}!");
        }

        private void tbScanSample_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                if (_client == null)
                {
                    MessageBox.Show("请先登录.");
                    return;
                }
                if (_currentLocation == null)
                {
                    MessageBox.Show("未获取当前位置.");
                    return;
                }
                //if (_currentPackage == null)
                //{
                //    MessageBox.Show("未获取当前箱号.");
                //    return;
                //}
                //if (_currentPackage.Status != (int)PackageStatus.Full)
                //{
                //    outputMessage($"当前箱子为{(_currentPackage.Status == (int)PackageStatus.Inbound ? "已入库" : "空闲")}状态,无法继续操作!");
                //    return;
                //}
                samplePreAssigning(null, null, tbScanSample.Text);
            }
        }

        private void btnScanRfid_Click(object sender, EventArgs e)
        {
            //if (!isConnected || reader == null)
            //{
            //    MessageBox.Show("读写器未连接,请先连接读写器!");
            //    return;
            //}
            //reader.ReadTag(currentSetting.btReadId, 2, 0, 8);
            if (!isConnected || reader == null)
            {
                if (!connectReader())
                {
                    MessageBox.Show("读写器未连接,请查看是否读写器设备!");
                    return;
                }
                Thread.Sleep(1000);
            }
            if (currentInventoryBuffer.bLoopInventory)
            {
                btnRealTimeInventory_Stop();
            }
            else
            {
                btnRealTimeInventory_Start();
            }
        }

        private void btnSampleAssign_Click(object sender, EventArgs e)
        {
            if (_currentScanSamples.IsNullOrEmpty()) return;
            var assignSamples = _currentScanSamples.ToList();//new一个list不影响原来的数组
            preSampleAssigning(assignSamples);
        }

        private void dgvSamples_RowStateChanged(object sender, DataGridViewRowStateChangedEventArgs e)
        {
            e.Row.Selected = false;
        }

        private void dgvRfidSamples_RowStateChanged(object sender, DataGridViewRowStateChangedEventArgs e)
        {
            e.Row.Selected = false;
        }

        private void dgvSamples_DataBindingComplete(object sender, DataGridViewBindingCompleteEventArgs e)
        {
            for (int i = 0; i < dgvSamples.Rows.Count; i++)
            {
                var item = dgvSamples.Rows[i].DataBoundItem as Sample;
                if (item.IsMatched)
                {
                    dgvSamples.Rows[i].DefaultCellStyle.BackColor = Color.Green;
                }
            }
        }

        private void ckbSelectAll_CheckedChanged(object sender, EventArgs e)
        {
            if (_currentScanSamples.IsNullOrEmpty()) return;
            this.Invoke(new Action(delegate
            {
                dgvRfidSamples.DataSource = null;
            }));
            foreach (var item in _currentScanSamples)
            {
                if (item.PackageKvid != Guid.Empty) continue;
                item.IsSelected = ckbSelectAll.Checked;
            }
            this.Invoke(new Action(delegate
            {
                dgvRfidSamples.DataSource = _currentScanSamples;
            }));
        }
    }
}
